# Mina Monitor - Что, как?
В данном **HOW-TO** мы рассмотрим различные варианты настройки Монитора. 
Будут рассмотрены только вопросы настройки портов и ip адресов и где и что запускать. 

### Сокращения и определения
- `Mina` - развернутый узел валидатора Мины
- `Монитор` - расширенный графический вариант команды `mina client status` с дополнительными показателями
- `узел валидатора` || `node` - сервер на котором установлен и запущен софт Mina  
- `SS` - серверная часть
- `Monitor SS` || `Сервер` - серверная часть Монитора
- `CS` - клиентская часть
- `Monitor CS` || `Клиент` - клиентская часть Монитора
- `host` - сервера и/или IP:PORT сервера на котором работает Mina
- `graphql` - GraphQL который создается и запускается узлом Mina

## Вариант 1 - Локальный клон
В первом варианте мы рассмотрим случай, когда все работает на одном сервере (Mina, Monitor SS, Monitor CS) и не выходит за пределы локальной сети.
Клонирование репозитория также выполнено на этот же сервер. Ничего и никуда больше копировать не планируется и смотреть монитор мы так же будем на этом компьютере.

### Конфигурация Сервера (Файл `server/config.js`)
Необходимо учесть, что Mina занимает, после запуска, следующие IP (интерфейсы) и порты:
- `внешний_IP:8302` - P2P адрес
- `localhost:3085` - GraphQL адрес

Будем считать, что в системе свободен интерфейс `localhost:8000` (вы можете использовать любой другой свободный порт выше 1024).
Будем использовать этот адрес для запуска Сервера. Для этого пропишем в конфигурационном файле следующие параметры для Сервера и GraphQL:

```json
{
    "host": "localhost:8000",
    "graphql": "localhost:3085"
}
```

Остальные параметры вы указываете на ваше усмотрение, согласно их описания в README. И это все, что касается настройки Серверной части Монитора.

### Конфигурация Клиента
Наш Клиент должен знать, где его серверная часть живет, что бы спрашивать у нее текущие показатели. Ранее мы указали параметр `host` для Сервера.
Это и есть требуемое значение. Опишем наш Сервер в параметре `hosts` и определим его использование в параметре `useHost`:
```json
{
    "hosts": {
        "node1": "localhost:8000"
    },
    "useHost": "node1"
}
```

Остальные параметры вы указываете на ваше усмотрение, согласно их описания в README. И это все, что касается настройки Серверной части Монитора.

### Запуск
Перейдем в каталог, в который мы клонировали репозиторий и выполним запуск.

**Для запуска сервера выполните команду:**
```shell
node server/monitor.mjs
```
В результате вы должны увидеть вывод следующего содержания без каких либо дополнительных сообщений:
```shell
Mina Node Server Monitor is running on http://localhost:8000
```

**Для запуска клиента выполните команду**
Для windows:
```shell
npm run serve
```
Для linux:
```shell
npm run serve_x
```
Запустится браузер, а в консоли вы должны увидеть вывод:
```shell
> @olton/mina-monitor@1.0.1 serve
> rimraf -rf output & md output & xcopy client\config.json ou
tput\*.* /Y && parcel serve --open -d output -p 2222 client/i
ndex.html

client\config.json
File copied: 1.
Server running at http://localhost:2222 
√  Built in 1.52s.
```

Если нужно запустить Клиента на другом порту (отличном от 2222), внесите изменения в команду запуска в файле `package.json` в разделе `scripts`.

## Вариант 2 - Локально + Web server
Если у вас на сервере запущен **web server**, вы можете запускать клиента из под него. Для этого выполняем все предыдущие шаги Варианта 1, кроме запуска клиента.
Во втором случае нам понадобится выполнить компиляцию файлов клиента для web сервера.
Для компиляции (сборки) Клиента выполните следующую команду:

Для windows
```shell
npm run build
```

Для linux
```shell
npm run build_x
```

При выполнении вы должны увидеть приблизительно такой вывод в консоли:

```shell
> @olton/mina-monitor@1.0.1 build
> rimraf -rf dist & md dist & npm run build_site


> @olton/mina-monitor@1.0.1 build_site
> xcopy client\config.json dist\*.* /Y /I && parcel build -d dist --public-url . --no-m
inify client/index.html

client\config.json
Files cpoied: 1.
√  Built in 1.18s.

dist\metro.05f88ce9.js.map         ‼  2.76 MB    240ms
dist\metro-all.86a093ae.css.map    ‼  1.47 MB     56ms
dist\metro.05f88ce9.js             ‼  1.11 MB    561ms
dist\metro-all.86a093ae.css        ‼  1.06 MB    148ms
dist\metro.7b7139cf.svg             536.33 KB    242ms
dist\chart.156ec6ff.js.map          205.25 KB    269ms
dist\metro.937a548f.woff            162.91 KB     28ms
dist\metro.45a8e540.ttf             162.84 KB    241ms
dist\chart.156ec6ff.js              107.93 KB     66ms
dist\app.173203a0.js.map              83.4 KB      8ms
dist\app.173203a0.js                 80.67 KB    345ms
dist\logo.d9bb64d7.png                51.2 KB    251ms
dist\index.html                      14.69 KB     12ms
dist\css.e1531409.css.map             4.42 KB      8ms
dist\css.e1531409.css                  2.8 KB     32ms
```

Теперь в папке `dist` находятся подготовленные для web сервера файлы и их можно скопировать в папку на вашем web сервере.
И запустить, набрав в браузере адрес клиента:
```shell
http://your_web_server_address/monitor_folder/
```

Обратите внимание, что в данный момент клиент работает исключительно по протоколу `http` по `https` клиент может не работать из-за смешанного содержания, так как Сервер Монитора запускается на протоколе `http` (в Chrome точно).

## Вариант 3 - Сервер и Клиент на разных машинах, Сервер работает рядом с Mina

В отличие от первых двух вариантов, этот вариант требует, что бы Сервер был запущен на открытом внешнем интерфейсе.
Исходники можно клонировать на любой компьютер.

### Конфигурация Сервера
Предположим, что у нас на сервере открыт интерфейс наружу с IP адресом `123.123.123.123` и портом `8000`.
Для этого пропишем в конфигурационном файле следующие параметры для Сервера и GraphQL (так как Сервер работает на том же компьютере, что и Mina, для доступа к GraqhQL используем localhost):
```json
{
    "host": "123.123.123.123:8000",
    "graphql": "localhost:3085"
}
```

### Конфигурация Клиента
```json
{
    "hosts": {
        "node1": "123.123.123.123:8000"
    },
    "useHost": "node1"
}
```

### Запуск Сервера
Скопируйте файлы из папки `server` в удобную для вас папке на севере Mina, перейдите в эту папку и выполните команду:
```shell
node server/monitor.mjs
```

### Запуск клиента
Если у вас нет web server, выполните из папки клонированного репозитория команду:
Для windows:
```shell
npm run serve
```
Для linux:
```shell
npm run serve_x
```

Или же, если у вас есть web сервер, выполните действия описанные для Варианта 2.

## Вариант 4 - Сервер и Клиент на разных машинах, Сервер работает отдельно от Mina
Сразу стоит сказать, что при таком варианте датчики Монитора **память** и **процессор** будут показывать состояние того компьютера, на котором запущена серверная часть монитора.

### Сервер
Копируем файлы из папки `server` репозитория в удобное для вас место на компьютере, где будет работать серверная часть Монитора.

Далее у нас два варианта:
1) Мы открываем порт на сервере Mina для доступа к GraphQL для серверной части Монитора
2) Мы пробрасываем порт GraphQL средствами SSH от сервера Mina на сервер Монитора
3) Мы пробрасываем порт Monitor Server средствами SSH от сервера Mina на ваш локальный компьютер

**Вариант 1**
Открываем порт GraphQL (флаг `-insecure-rest-server` при запуске Mina). 
Пусть ip адрес узла Mina 1.1.1.1 и порт GraphQL 3085, а ip адрес Сервера Монитора и порт - 2.2.2.2:8000 
В конфигурационном файле Сервера прописываем:
```json
{
    "host": "2.2.2.2:8000",
    "graphql": "1.1.1.1:3085"
}
```
Запускаем Сервер папке куда скопировали файлы сервера командой:
```shell
node monitor.mjs
```

**Вариант 2**
Пробрасываем GraphQL командой
```shell
ssh -L 3085:localhost:3085 user@mina_server_ip_address
```
Теперь у нас локально, по адресу `localhost:3085` появился работающий сервер GraphQL (проверить можно перейдя по ссылке `http://localhost:3085/graphql`)
Пусть ip адрес Сервера Монитора и порт - 2.2.2.2:8000
В конфигурационном файле Сервера прописываем:
```json
{
    "host": "2.2.2.2:8000",
    "graphql": "localhost:3085"
}
```
Запускаем Сервер папке куда скопировали файлы сервера командой:
```shell
node monitor.mjs
```

### Клиент
В конфигурационном файле клиента прописываем
```json
{
    "hosts": {
        "node1": "2.2.2.2:8000"
    },
    "useHost": "node1"
}
```

Компилируем и запускаем любым из вариантов.

**Вариант 3**
Пусть вы запустили сервер монитора на компьютере с ip адресом `1.1.1.1` и портом `8000`.
В конфигурации сервера указано

```json
{
    "host": "1.1.1.1:8000"
}
```

Пробрасываем Monitor Server командой
```shell
ssh -L 8000:1.1.1.1:8000 user_name@1.1.1.1
```
После выполнения этой команды у вас локально на порту 8000 будет работать серверная часть монитора.
Теперь вы можете указать в настройках:

Для Клиента:

```json
{
    "hosts": {
        "node1": "localhost:8000"
    }
}
```
